<?php
declare (strict_types = 1);

namespace app\command;

use think\console\Command;
use think\console\Input;
use think\console\input\Argument;
use think\console\input\Option;
use think\console\Output;
use Workerman\worker;
use Workman\Connection\TcpConnection;

require __DIR__.'/../../vendor/autoload.php';

class Webchat extends Command
{
    protected function configure()
    {
        // 指令配置
        $this->setName('startWebchat')
            ->setDescription('the startWebchat command');
    }

    protected function execute(Input $input, Output $output)
    {
        // 指令输出
        $output->writeln('startWebchat');

        // 开启一个websocket服务，该端口为前端请求的端口
        $ws_worker = new Worker('websocket://0.0.0.0:2888');
        $ws_worker->count = 4;
        $ws_worker->name='test';

        $ws_worker->onWorkerStart = function() {

            // 开启一个内部端口，方便内部系统推送数据，Text协议格式 文本+换行符
            // 开启内部端口的目的是方便服务端在某个时刻需要向前端推数据，提供了一个推数据的通道，这里的ip/端口号和前端没有关系，正常情况下是为了实现数据突变上送

            $inner_text_worker = new Worker('text://0.0.0.0:2346');

            // 这里是收集服务端本身向这个ws服务发数据时随时接受，然后处理完成后返回给前端
            $inner_text_worker->onMessage = function($connection, $buffer) {
                global $ws_worker;
                // $data数组格式，里面有uid，表示向那个uid的页面推送数据
                $data = json_decode($buffer, true);
                if(!empty($data['uid'])) {
                    $uid = $data['uid'];
                    // 通过workerman，向uid的页面推送数据
                    $ret = $this->sendMessageByUid($ws_worker,$uid, $buffer);
                } else {
                    // 广播消息
                    $ret = $this->broadcast($ws_worker,$buffer);
                }

                // 返回推送结果给请求这个地址的服务端
                $connection->send($ret ? 'ok' : 'fail');
            };

            // ## 执行监听 ##

            $inner_text_worker->listen();

        };

        // 新增加一个属性，用来保存uid到connection的映射
        $ws_worker->uidConnections = array();

        // 当有客户端发来消息时执行的回调函数
        $ws_worker->onMessage = function($connection, $data) {

            global $ws_worker;

            // 判断当前客户端是否已经验证,既是否设置了uid

            if(!isset($connection->uid))
            {
                // 没验证的话把第一个包当做uid（这里为了方便演示，没做真正的验证）
                $connection->uid = $data;
                /* 保存uid到connection的映射，这样可以方便的通过uid查找connection，

                               * 实现针对特定uid推送数据

                               */
                $ws_worker->uidConnections[$connection->uid] = $connection;

            }
            // 这句话表示客户端有请求来时，立即给客户端返回数据，正常情况下这里要根据客户端传的参数，做不同处理，返回不同数据
            $connection->send('receive msg');

        };

        // 当有客户端连接断开时，断开删除已连接的客户端请求
        $ws_worker->onClose = function($connection) {
            global $ws_worker;

            if(isset($connection->uid)) {
                // 连接断开时删除映射
                unset($ws_worker->uidConnections[$connection->uid]);
            }
        };

        worker::runAll();
    }

    // 针对uid推送数据
    function sendMessageByUid($worker,$uid, $message) {

        if(isset($worker->uidConnections[$uid])) {
            $connection = $worker->uidConnections[$uid];
            $connection->send($message);
            return true;
        }

        return false;

    }

    // 向所有验证的用户推送数据
    function broadcast($worker,$message) {
        foreach($worker->uidConnections as $connection) {
            $connection->send($message);
        }
        return true;
    }
}