<?php
namespace app\work\controller;
use think\facade\Db;
use think\worker\Server;
use Workerman\Lib\Timer;
use Workerman\worker;
use think\facade\View;

/*
 * 业务需求示例：系统后台监听有新的商品订单，新消息等的通知，推送给所有登陆系统的用户。
逻辑：使用websocket建立连接，并设置定时器发送心跳保持连接不被断开。建立连接后，php端需要设置心跳时间，判断mysql业务表是否有新记录或者订单，如果有就推送消息给所有登陆系统用户，并更新该记录已推送。
 */
class Push extends Server
{
    protected $socket = 'http://0.0.0.0:2346';   //端口自行修改

    protected static $heartbeat_time = 55;
    public $connectionArr = array();

    public function onWorkerStart($worker)
    {

        // 开启一个内部端口，方便内部系统推送数据，Text协议格式 文本+换行符
        // 开启内部端口的目的是方便服务端在某个时刻需要向前端推数据，提供了一个推数据的通道，这里的ip/端口号和前端没有关系，正常情况下是为了实现数据突变上送

        $inner_text_worker = new Worker('text://0.0.0.0:5678');

        // 这里是收集服务端本身向这个ws服务发数据时随时接受，然后处理完成后返回给前端
        $inner_text_worker->onMessage = function($connection, $buffer) {
            //global $worker;
            // $data数组格式，里面有uid，表示向那个uid的页面推送数据
            $data = json_decode($buffer, true);
            if(!empty($data['uid'])) {
                $uid = $data['uid'];
                // 通过workerman，向uid的页面推送数据
                $ret = $this->sendMessageByUid($uid, $buffer);
            } else {
                // 广播消息
                $ret = $this->broadcast($buffer);
            }

            // 返回推送结果给请求这个地址的服务端
            $connection->send($ret ? 'ok888' : 'fail666');
        };

        // ## 执行监听 ##
        $inner_text_worker->listen();


        // 进程启动后设置一个每秒运行一次的定时器
        //查看是否有新的充值或提现订单，有就推送给所有用户
        Timer::add(3, function () use ($worker) {

            $time_now = time();


            //是否有新消息
            $count = 10;

            if ($count > 0) {



                foreach ($worker->connections as $connection) {

                    // 有可能该connection还没收到过消息，则lastMessageTime设置为当前时间
                    if (empty($connection->lastMessageTime)) {
                        $connection->lastMessageTime = $time_now;
                        continue;
                    }

                    if((int)$connection->uid !== 10000){
                        $connection->send("当前用户id非10000");
                    }else{
                        $data =  [
                            'toUser' => 10000,
                            'msgContent' => 'success6666',
                            'connection' => json($connection)
                        ];
                        //$this->sendMsg($data);
                    }


                    //$connection->lastMessageTime 是指上次通讯时间
                    //// 上次通讯时间间隔大于心跳间隔，则认为客户端已经下线，关闭连接
                    if ($time_now - $connection->lastMessageTime > self::$heartbeat_time) {
                        //后台控制连接
                        //前端断了 可以点击重新连接 类似游戏重新登录
                        $connection->close();
                    }

                    $data = ['time'=> $time_now,'lastMessageTime' => $connection->lastMessageTime];

                    //$connection->send(json_encode($data));
                }
                //处理发送状态 ['is_push' => 1]
            } else {
                foreach ($worker->connections as $connection) {
                    if (empty($connection->lastMessageTime)) {
                        $connection->lastMessageTime = $time_now;
                        continue;
                    }

                    if ($time_now - $connection->lastMessageTime > self::$heartbeat_time) {      //连接超时
                        $connection->close();
                    }
                }
            }
        });
    }

    // 针对uid推送数据
    function sendMessageByUid($uid, $message)
    {
        if (isset($this->connectionArr[$uid])) {
            $connection = $this->connectionArr[$uid];
            $connection->send($message);
            return true;
        }
    }

    // 向所有验证的用户推送数据
    function broadcast($message) {
        foreach($this->connectionArr as $connection) {
            $connection->send($message);
        }
        return true;
    }

    /**
     * 收到信息
     * @param $connection
     * @param $data
     */
    public function onMessage($connection, $data)
    {
        //收到消息刷新最后时间 前端发送心跳
        $connection->lastMessageTime = time();
        $wsData = json_decode($data,true);

        if(!isset($wsData['option'])){
            $connection->send('接收数据:'.$data);
            return;
        }
        switch($wsData['option']){
            case "login":
                $connection->uid=$wsData['uid'];
                //echo $wsData['uid']."上线了";
                $connection->send($wsData['uid']."上线了");
                $this->setOnline($connection);
                break;
            default:
                $connection->send('其他数据');
                break;
        }
    }

    //如果是发送消息
    public function sendMsg($data){
        /*$data =  [
            'toUser' => 10000,
            'msgContent' => '主动发送',
        ];*/
        $msgStr = json_encode(array("option"=>"sendMsg","msg"=>"发给".$data['toUser']."的消息:".$data['msgContent']));
        $this->connectionArr[$data['toUser']]->send($msgStr);
    }

    //用户上线
    public function setOnline($connection){
        $this->connectionArr[$connection->uid] = $connection;
    }

    /**
     * 当连接建立时触发的回调函数
     * @param $connection
     */
    public function onConnect($connection)
    {
    }

    /**
     * 当连接断开销毁连接对象节约内存空间
     * @param $connection
     */
    public function onClose($connection)
    {
        // 连接断开时删除映射
        unset($this->connectionArr[$connection->uid]);
    }

    /**
     * 当客户端的连接上发生错误时触发
     * @param $connection
     * @param $code
     * @param $msg
     */
    public function onerror($connection, $code, $msg)
    {
        echo "\r\n error $code $msg\n";
    }

    /*
     * 阿里云
     * https://ecs.console.aliyun.com/
     * kod桌面
     * http://rong.zx2049.com/
     * 禅道
     * http://zentao.dev.zx2049.com/
     * 宝塔
     * https://47.120.56.77:25705/
     * layui手册
     * http://layui.apixx.net/demo/panel.html
     * tp5.1手册
     * https://www.kancloud.cn/manual/thinkphp5_1/354084
     * tp8.0手册
     * https://doc.thinkphp.cn/v8_0/shitubianliang.html
     * 后端
     * http://console.zx2049.com/admin/index/index.html#
     * 客户端
     * http://workerman.dev.zx2049.com/work/push/index
     */
    public function index(){

        //获取规则
        $rule = ['默认规则'];
        for ($i = 1; $i <= 10; $i++) {
            $rule[] = '规则'.$i;
        }
        View::assign('rule',$rule);
        return View::fetch();
    }

    /*
     * 测试
     * http://workerman.dev.zx2049.com/work/push/test
     */
    public function test(){

        $post = request()->post();
        if(empty($post['info'])){
            return false;
        }
        // 建立socket连接到内部推送端口
        $client = stream_socket_client('tcp://127.0.0.1:5678', $errno, $errmsg, 1);
        //dump($client);
        // 推送的数据，包含uid字段，表示是给这个uid推送
        $data = array('uid'=>'10000', 'percent'=>'88%','msg'=>$post['info']);

        // 发送数据，注意5678端口是Text协议的端口，Text协议需要在数据末尾加上换行符
        fwrite($client, json_encode($data,JSON_UNESCAPED_UNICODE)."\n");

        // 读取推送结果
        echo fread($client, 8192);
    }

}